package jetbrains.sample.statistics;

import com.intellij.util.containers.HashMap;
import jetbrains.buildServer.serverSide.SBuild;
import jetbrains.buildServer.serverSide.SBuildServer;
import jetbrains.buildServer.serverSide.SBuildType;
import jetbrains.buildServer.serverSide.statistics.ValueTypeRegistry;
import jetbrains.buildServer.serverSide.statistics.ValueFilter;
import jetbrains.buildServer.serverSide.statistics.build.*;
import jetbrains.buildServer.util.filters.Filter;
import jetbrains.buildServer.util.filters.FilterUtil;
import jetbrains.buildServer.vcs.SVcsModification;
import jetbrains.buildServer.vcs.SelectPrevBuildPolicy;
import jetbrains.buildServer.web.openapi.SimpleWebExtension;
import jetbrains.buildServer.web.openapi.WebControllerManager;
import jetbrains.buildServer.web.openapi.WebPlace;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.util.*;

/**
 * Sample statistics data provider
 */
public class SampleStatisticsValue implements BuildValueType, BuildFinishAware {

    private final ValueTypeRegistry myValueTypeRegistry;
    protected SBuildServer myServer;
    private final WebControllerManager myWebControllerManager;
    protected BuildDataStorage myStorage;

    public SampleStatisticsValue(final BuildDataStorage storage, final ValueTypeRegistry valueTypeRegistry,
                                 final SBuildServer server, WebControllerManager manager) {
        myStorage = storage;
        myValueTypeRegistry = valueTypeRegistry;
        myServer = server;
        myWebControllerManager = manager;
    }
    
    public void register(){
        myValueTypeRegistry.registerValueType(this);
        SimpleWebExtension statExt = new SimpleWebExtension(myWebControllerManager);
        statExt.setJspPath("sampleStatistics.jsp");
        statExt.setPlace(WebPlace.BUILD_TYPE_STATISTICS_FRAGMENT);
        statExt.setPluginName("samplePlugin");
        statExt.setTitle("Sample Statistics Title");
        statExt.register();
        
    }

    /**
     * Returns statistics data stored in general storage. Some other storage can be used here.
     * Also ValueType can compose result value from several other ValueTypes registered in system. 
     * @param valueFilter filter which has to be applyed to returned values.
     * @return iterable collection of statistics values.
     */
    @NotNull
    public List<BuildValue> getDataSet(@NotNull ValueFilter valueFilter) {
        jetbrains.sample.statistics.SampleBuildValueFilter filter = (jetbrains.sample.statistics.SampleBuildValueFilter) valueFilter;
        final Set<String> selectedUsers = filter.getSelectedUsers();
        List<BuildValue> entireCollection = myStorage.getDataSet(getKey(), filter, BuildValueTransformer.ourDEFAULT);
        if (!selectedUsers.isEmpty()) {
            return FilterUtil.filterCollection(new ArrayList<BuildValue>(entireCollection),
                    createByUsersFilter(selectedUsers));
        }
        else {
            return entireCollection;
        }
    }

    private Filter<BuildValue> createByUsersFilter(final Set<String> selectedUsers) {
        return new Filter<BuildValue>() {
            public boolean accept(@NotNull BuildValue data) {
                return checkBuildContainsChangeAtLeastOneUserFrom(data, selectedUsers);
            }
        };
    }

    private boolean checkBuildContainsChangeAtLeastOneUserFrom(BuildValue data, Set<String> selectedUsers) {
        SBuild build = myServer.findBuildInstanceById(data.getBuildId());
        if (build != null) {
            List<SVcsModification> changes = build.getContainingChanges();
            for (SVcsModification change : changes) {
                if (selectedUsers.contains(change.getUserName())) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Unique statistics identificator.
     * @return unique string key.
     */
    @NotNull
    public String getKey() {
        return "SampleStatistics";
    }

    /**
     * Returns filter based on given parameters
     * @param stringMap
     * @return
     */
    public BuildValueFilter createFilter(Map<String, String[]> stringMap) {
        HashMap<String, String[]> values = new HashMap<String, String[]>();
        values.putAll(stringMap);
        SampleBuildValueFilter resultFilter = new jetbrains.sample.statistics.SampleBuildValueFilter(values);
        resultFilter.setAvailableUsers(getAllCommitterNames(stringMap));
        resultFilter.setAvailableAgents(myStorage.getAgents(resultFilter));
        
        resultFilter.update(stringMap);
        return resultFilter;
    }

    private Set<String> getAllCommitterNames(Map<String, String[]> stringMap) {
        String[] buildTypeId = stringMap.get("buildTypeId");
        Set<String> availableUsers = new HashSet<String>();
        if (buildTypeId != null) {
            SBuildType buildType = myServer.getProjectManager().findBuildTypeById(buildTypeId[0]);
            if (buildType != null) {
                List<SVcsModification> allModifications = myServer.getVcsHistory().getAllModifications(buildType);
                availableUsers.addAll(getAllUserNames(allModifications));
            }
        }
        return availableUsers;
    }

    /**
     * Includes each user who made modification in the given modification list
     * @param allModifications
     * @return
     */
    private Set<String> getAllUserNames(List<SVcsModification> allModifications) {
        Set<String> names = new HashSet<String>();
        for (SVcsModification modification : allModifications) {
            if (modification.getUserName() != null) {
                names.add(modification.getUserName());
            }
        }
        
        return names;
    }

    @Nullable
    public NumberFormat getValueFormat() {
        return NumberFormat.getIntegerInstance();
    }

    /**
     * Publishes build statistics
     * @param sBuild
     */
    public void buildFinished(SBuild sBuild) {
        myStorage.publishValue(getKey(), sBuild.getBuildId(), new BigDecimal(sBuild.getChanges(SelectPrevBuildPolicy.SINCE_LAST_BUILD, true).size()));
    }

    /**
     * Statistics description. Placed above the graph 
     * @param valueFilter current filter
     * @return
     */
    @NotNull
    public String getDescription(ValueFilter valueFilter) {
        Set<String> selectedUsers = ((jetbrains.sample.statistics.SampleBuildValueFilter) valueFilter).getSelectedUsers();
        if (selectedUsers.size() != 1) {
            return "Sample Statistics Description";
        }
        else {
            return "Sample Statistics for user " + selectedUsers.iterator().next(); 
        }
    }

    @NotNull
    public String getExtendedDescription(ValueFilter currentFilter) {
        return "Extended Description";
    }

}
